#include <MailTab.h>
#include <hwTableLayout.h>
#include <hwScrollWindow.h>
#include <hwTable.h>
#include <hwButtonbar.h>
#include <hwLabel.h>
#include <hwText.h>
#include <hwTree.h>
#include <ComposeDialog.h>
#include <hwWindow.h>
#include <hwCheckbox.h>
#include <stdlib.h>
#include <hwCombo.h>

class messageTable : public hwTable
{
public:
    messageTable(MailTab* d) : myMailTab(d) {}
    void onClick()
    {
        myMailTab->showMessage();
    }
private:
    MailTab* myMailTab;
};

class mailButtonbar : public hwButtonbar
{
public:
    mailButtonbar(MailTab* d) : myMailTab(d) {}
    void onClick()
    {
        if (getStringValue() == "New")
        {
            myMailTab->createNewMessage();
        }
    }
private:
    MailTab* myMailTab;
};

class mailTree : public hwTree
{
public:
    mailTree(MailTab* d) : myMailTab(d) {}
    void onClick()
    {
        int id = atoi(myValue.c_str());
        hwWidget* w = getNode(id);
        if (w)
        {
            myMailTab->updateTable(w->toString());
        }
    }
private:
    MailTab* myMailTab;
};

MailTab::MailTab(hwWindow* window)
        : myWindow(window)
{
    myComposeDialog = new ComposeDialog(myWindow);
    
    setName("mailTab");
    
    hwTableLayout* tablelayout1 = new hwTableLayout();
    add(tablelayout1);
    
    hwCombo* combo1 = new hwCombo();
    tablelayout1->add(combo1, 0, 0);
    
    hwScrollWindow* scrollwindow2 = new hwScrollWindow();
    tablelayout1->add(scrollwindow2, 1, 0);
    scrollwindow2->setName("mailFolderScrollWindow");
    myFolderTree = new mailTree(this);
    scrollwindow2->add(myFolderTree);
    
    hwTableLayout* tablelayout2 = new hwTableLayout();
    add(tablelayout2);
    mailButtonbar* buttonbar1 = new mailButtonbar(this);
    buttonbar1->add("New");
    buttonbar1->add("Delete");
    buttonbar1->add("Reply");
    buttonbar1->add("Reply All");
    buttonbar1->add("Forward");
    buttonbar1->add("Junk");
    tablelayout2->add(buttonbar1, 0, 0);
    
    hwScrollWindow* scrollwindow1 = new hwScrollWindow();
    scrollwindow1->setName("mailMessageScrollWindow");
    tablelayout2->add(scrollwindow1, 1, 0);
    myMessageTable = new messageTable(this);
    myMessageTable->setColumnLabel(0, new hwLabel(""));
    myMessageTable->setColumnLabel(1, new hwLabel("Subject"));
    myMessageTable->setColumnLabel(2, new hwLabel("From"));
    myMessageTable->setColumnLabel(3, new hwLabel("Size"));
    myMessageTable->setColumnLabel(4, new hwLabel("Received"));
    scrollwindow1->add(myMessageTable);
    
    myMessageText = new hwText();
    myMessageText->setName("mailText");
    tablelayout2->add(myMessageText, 2, 0);
    
    loadFolders();
    loadMessages();
}

void MailTab::loadFolders()
{
    hwLabel* label1 = new hwLabel("Folders");
    myFolderTree->addNode(label1, new hwLabel("Inbox"));
    myFolderTree->addNode(label1, new hwLabel("Draft"));
    myFolderTree->addNode(label1, new hwLabel("Sent"));
    myFolderTree->addNode(label1, new hwLabel("Trash"));
    myFolderTree->addNode(label1, new hwLabel("Junk"));
    
    vector<Message> v;
    
    myFolderMap["Inbox"] = v;
    myFolderMap["Draft"] = v;
    myFolderMap["Sent"] = v;
    myFolderMap["Trash"] = v;
    myFolderMap["Junk"] = v;
}

void MailTab::loadMessages()
{
    myFolderMap["Inbox"].push_back(
        Message("How's it going?", "bob@gmail.com", "100KB", "Feb 19 10:00am",
            "Good to hear from you!"));
    myFolderMap["Inbox"].push_back(
        Message("Buy a new car", "joe@gmail.com", "120KB", "Feb 19 8:00am",
        "For a limited time buy a new car for $5!!!"));
    myFolderMap["Inbox"].push_back(
        Message("Lunch", "paul@gmail.com", "10KB", "Feb 19 7:00am",
        "Hey dude, want to grab lunch today?"));
    myFolderMap["Inbox"].push_back(
        Message("Ebay Discounter", "junk@gmail.com", "1000KB", "Feb 19 6:00am",
        "Make a million on Ebay."));
    
    myFolderMap["Junk"].push_back(
        Message("Ebay Discounter Again", "junk@gmail.com", "1100KB", "Feb 19 5:00am",
        "Princess in Zimbabwe needs your money"));
    myFolderMap["Junk"].push_back(
        Message("Viagra", "money@gmail.com", "100KB", "Feb 18 5:00am",
        "Need I say more!"));
}

void MailTab::updateTable(const string& folder)
{
    myCurrentFolder = folder;
    
    vector<Message> v = myFolderMap[folder];
    
    myMessageTable->clear();
    
    for (unsigned int i = 0; i < v.size(); i++)
    {
        cout << v[i].subject << endl;
        myMessageTable->setCellValue(i, 0, new hwCheckbox(""));
        myMessageTable->setCellValue(i, 1, new hwLabel(v[i].subject));
        myMessageTable->setCellValue(i, 2, new hwLabel(v[i].from));
        myMessageTable->setCellValue(i, 3, new hwLabel(v[i].size));
        myMessageTable->setCellValue(i, 4, new hwLabel(v[i].received));
    }
}

void MailTab::showMessage()
{
    int row = atoi(myMessageTable->getValue().c_str());
    vector<Message> v = myFolderMap[myCurrentFolder];
    myMessageText->setValue(v[row].message);
}

void MailTab::createNewMessage()
{
    myComposeDialog->show();
}

