/*
 * Copyright (c) 2009 Adam Siegel.  All rights reserved.
 *
 * This file is part of libhw.
 *
 * libhw is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with libhw.  If not, see <http: *www.gnu.org/licenses/>.
 *
 */

#ifndef HWAPP_H
#define HWAPP_H

#include <map>
#include <string>

using namespace std;

class hwWindow;
class hwTheme;
class hwWidget;
class wsRequest;

/**
 * This class.
 *
 */
class hwApp
{
public:

    hwApp();

    ~hwApp();

    /**
     * As widgets are created, they are registered with
     * their application instance.  This is required to
     * create hash tables for fast lookup of widgets
     * as they need to be processed by incoming requests.
     */
    void registerWidget(hwWidget* w);

    /**
     * When a connection is received by the server,
     * it is converted into a request object and sent
     * to the application for processing.
     */
    void process(wsRequest* request);

    virtual hwApp* createInstance() = 0;

    virtual void init() = 0;

    void setActiveWindow(hwWindow* window)
    {
        myActiveWindow = window;
    }

    void setTheme(hwTheme* theme)
    {
        myTheme = theme;
    }

    void setId(const string& id)
    {
        myId = id;
    }

    hwTheme* getTheme() const
    {
        return myTheme;
    }

    string getId() const
    {
        return myId;
    }

private:

    hwWindow* myActiveWindow;
    hwTheme* myTheme;
    map<string,hwWidget*> myWidgetMap;
    string myId;
    string myClass;
};

#endif

