/*
 * Copyright (c) 2009 Adam Siegel.  All rights reserved.
 *
 * This file is part of libhw.
 *
 * libhw is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with libhw.  If not, see <http: *www.gnu.org/licenses/>.
 *
 */

#ifndef HWWIDGET_H
#define HWWIDGET_H

#include <string>
#include <iostream>
#include <pthread.h>

using namespace std;

class hwServer;

class hwWidget
{
public:

    virtual ~hwWidget();

    virtual string draw() = 0;

    string getName() const
    {
        return myName;
    }

    void setValue(const string& value)
    {
        myValue = value;
    }

    void setName(const string& name);

    string getValue() const
    {
        return myValue;
    }

    bool isVisible() const
    {
        return myVisibleFlag;
    }

    void setVisibility(bool visible)
    {
        myVisibleFlag = visible;
    }

    string getClass()
    {
        return myClass;
    }

    virtual string toString()
    {
        return "";
    }
    
    virtual void onClick() {};
    virtual void onChange() {};

protected:

    hwWidget(const string& classname);

    string myName;
    string myValue;
    string myClass;
    bool myVisibleFlag;

private:

    hwWidget(const hwWidget& source);
    hwWidget& operator=(const hwWidget& source);

    static pthread_mutex_t myMutex;
    static unsigned long myWidgetCounter;
};

#endif


