/*
 * Copyright (c) 2009 Adam Siegel.  All rights reserved.
 *
 * This file is part of libhw.
 *
 * libhw is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with libhw.  If not, see <http: *www.gnu.org/licenses/>.
 *
 */

#ifndef WS_REQUEST_H
#define WS_REQUEST_H

#include <string>
#include <map>
#include <vector>

using namespace std;

class wsRequest
{
public:

    wsRequest(int socket);

    ~wsRequest();
    
    void process();

    string getParam(const string& param)
    {
        return myParamVals[param];
    }

    void output(const string& data);

    void output(const char* data, const string& type, int length, int secs);
    
    vector<string> getParamList()
    {
        return myParamList;
    }

    void setCookie(const string& key, const string& val)
    {
        myResponseVals["Set-Cookie"] = key + "=" + val;
    }
    
    bool isResource()
    {
        return myResourceFlag;
    }
    
    void setResourceFlag(bool flag)
    {
        myResourceFlag = flag;
    }
    
    string getPath()
    {
        return myPath;
    }

private:

    void parseQuery(const string& query);
    
    string toLower(const string& s);

    string strip(const string& s);

    string decode(const string& s);
    
    void send(const char* data, int length);
    
    string getTime(time_t secs);

    map<string,string> myHeaderVals;
    map<string,string> myParamVals;
    map<string,string> myResponseVals;
    string myContent;
    vector<string> myParamList;
    int mySocket;
    bool myResourceFlag;
    string myPath;
};

#endif

