/*
 * Copyright (c) 2009 Adam Siegel.  All rights reserved.
 *
 * This file is part of libhw.
 *
 * libhw is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with libhw.  If not, see <http: *www.gnu.org/licenses/>.
 *
 */

#include <hwApp.h>
#include <hwWindow.h>
#include <hwServer.h>
#include <hwTheme.h>
#include <hwWidget.h>
#include <hwScrollWindow.h>
#include <hwSplitPane.h>
#include <vector>
#include <string>
#include <iostream>
#include <wsRequest.h>

hwApp::hwApp()
        : myClass("hwApp")
{
}

hwApp::~hwApp()
{
}

void hwApp::process(wsRequest* request)
{
    /* Update any values */
    
    vector<string> params = request->getParamList();
    
    for (int i = 0; i < params.size(); i++)
    {
        hwWidget* w = myWidgetMap[params[i]];
        if (w)
        {
            w->setValue(request->getParam(params[i]));
        }
    }
    
    /* Process any callbacks */
    
    string widget = request->getParam("_widget");
    hwWidget* w = myWidgetMap[widget];
    string event = request->getParam("_event");
    
    if (w && event.size())
    {
        cerr << "Processing event: " << event << " for " << w->getName() << endl;
        
        if (event == "click")
        {
            w->onClick();
        }
        else if (event == "change")
        {
            w->onChange();
        }
    }
    
    /* Generate output html */
    
    string html;
    
    if (request->getParam("_frame").size())
    {
        hwWidget* w = (hwScrollWindow*)myWidgetMap[request->getParam("_frame")];
        
        if (w->getClass() == "hwScrollWindow")
        {
            html = ((hwScrollWindow*)w)->drawChild();
        }
    }
    else
    {
        html = myActiveWindow->draw();
    }
    
    request->output(html);
}

void hwApp::registerWidget(hwWidget* w)
{
    if (w)
    {
        myWidgetMap[w->getName()] = w;
    }
}

