/*
 * Copyright (c) 2009 Adam Siegel.  All rights reserved.
 *
 * This file is part of libhw.
 *
 * libhw is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with libhw.  If not, see <http: *www.gnu.org/licenses/>.
 *
 */

#include <hwServer.h>
#include <stdlib.h>
#include <iostream>
#include <hwApp.h>
#include <sstream>
#include <time.h>
#include <pthread.h>
#include <md5.h>
#include <hwWidget.h>
#include <wsServer.h>
#include <wsRequest.h>
#include <hwResourceMgr.h>
#include <signal.h>

hwServer* hwServer::myInstance = NULL;

hwServer::hwServer()
        : myMD5(new MD5())
{
    pthread_mutex_init(&myMutex, NULL);
}

hwServer* hwServer::getInstance()
{
    if (!myInstance)
    {
        myInstance = new hwServer();
    }
    
    return myInstance;
}

void hwServer::run(unsigned int port)
{
#ifndef WIN32
    signal(SIGPIPE, SIG_IGN);
#endif

    wsServer* server = new wsServer(port);
    
    server->init();
    
    while (wsRequest* req = server->getRequest())
    {
        pthread_t th;
        pthread_create(&th, NULL, hwServer::process, req);
    }
}

void* hwServer::process(void* data)
{
    wsRequest* request = (wsRequest*)data;
    string hwid = request->getParam("hwid");
    hwApp* app;
    pthread_t thread = pthread_self();
    
    pthread_mutex_lock(&myInstance->myMutex);
    
    if (!myInstance->myAppMap[hwid])
    {
        time_t secs;
        time(&secs);
        srand(secs);
        stringstream s;
        s << rand();
        string key = myInstance->createKey(s.str());
        app = myInstance->myAppFactory->createInstance();
#ifdef WIN32
        myInstance->myThreadMap[(unsigned long)pthread_getw32threadhandle_np(thread)] = app;
#else
        myInstance->myThreadMap[thread] = app;
#endif
        app->setId(key);
        app->init();
        cerr << "Create App Instance " << key << endl;
        myInstance->myAppMap[key] = app;
        request->setCookie("hwid", key);
    }
    else
    {
        app = myInstance->myAppMap[hwid];
#ifdef WIN32
        myInstance->myThreadMap[(unsigned long)pthread_getw32threadhandle_np(thread)] = app;
#else
        myInstance->myThreadMap[thread] = app;
#endif
    }
    
    pthread_mutex_unlock(&myInstance->myMutex);
    
    if (request->isResource())
    {
        hwResourceMgr::getInstance()->process(request);
    }
    else
    {
        app->process(request);
    }

    delete request;
    
    myInstance->threadDone(thread);    
    
    pthread_exit(NULL);
}

hwApp* hwServer::getApp(pthread_t thread)
{
#ifdef WIN32
    return myThreadMap[(unsigned long)pthread_getw32threadhandle_np(thread)];
#else
    return myThreadMap[thread];
#endif
}

void hwServer::threadDone(pthread_t thread)
{
    pthread_mutex_lock(&myInstance->myMutex);
#ifdef WIN32
    myThreadMap.erase((unsigned long)pthread_getw32threadhandle_np(thread));
#else
    myThreadMap.erase(thread);
#endif
    pthread_mutex_unlock(&myInstance->myMutex);
}

string hwServer::createKey(const string& s)
{
    MD5_CTX ctx;
    myMD5->MD5Init(&ctx);
    myMD5->MD5Update(&ctx, (unsigned char*)s.c_str(), s.length());
    unsigned char buff[16] = "";
    myMD5->MD5Final((unsigned char*)buff, &ctx);
    char key[33];
    int p = 0;
    for (int i = 0; i < 16; i++)
    {
        sprintf(&key[p], "%02x", buff[i]);
        p += 2;
    }
    key[32] = '\0';
    return std::string(key);
}

