/*
 * Copyright (c) 2009 Adam Siegel.  All rights reserved.
 *
 * This file is part of libhw.
 *
 * libhw is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with libhw.  If not, see <http: *www.gnu.org/licenses/>.
 *
 */

#include <hwTable.h>
#include <hwLabel.h>
#include <sstream>

hwTable::hwTable()
        : hwWidget("hwTable")
{
}

hwTable::~hwTable()
{
}

string hwTable::draw()
{
    if (!myVisibleFlag)
    {
        return "";
    }
    
    string html;
    
    html += "<input type=\"hidden\" name=\"" + myName + "\">";
    html += "<table class=\"hwTable\" id=\"" + 
        myName + "\" name=\"" + myName + "\">";
        
    html += "<tr>";
        
    for (unsigned int i = 0; i < myCols.size(); i++)
    {
        html += "<th>";
        html += myCols[i] ? myCols[i]->draw() : "&nbsp;";
        html += "</th>";
    }
    
    html += "</tr>";
    
    for (unsigned int i = 0; i < myCells.size(); i++)
    {
        stringstream s;
        s << i;
        
        if (s.str() == myValue)
        {
            html += "<tr class=\"hwTableSelectedRow\" "; 
            html += "onClick=\"";
            html += "document.forms[0]._widget.value='" + myName + "';";
            html += "document.forms[0]._event.value='click';";
            html += "document.forms[0]." + myName + ".value='" + s.str() + "';"; 
            html += "document.forms[0].submit();\" ";
            html += "onMouseOver=\"this.className='hwTableHoverRow'\" ";
            html += "onMouseOut=\"this.className='hwTableSelectedRow'\" ";
            html += ">";
        }
        else
        {
            html += "<tr class=\"hwTableRow\" "; 
            html += "onClick=\"";
            html += "document.forms[0]._widget.value='" + myName + "';";
            html += "document.forms[0]._event.value='click';";
            html += "document.forms[0]." + myName + ".value='" + s.str() + "';"; 
            html += "document.forms[0].submit(); ";
            html += "top.location='/';\" ";
            html += "onMouseOver=\"this.className='hwTableHoverRow'\" ";
            html += "onMouseOut=\"this.className='hwTableRow'\" ";
            html += ">";
        }
        
        for (unsigned int j = 0; j < myCells[i].size(); j++)
        {
            html += "<td>";
            html += myCells[i][j] ? myCells[i][j]->draw() : "&nbsp;";
            html += "</td>";
        }
        
        html += "</tr>";
    }
    
    html += "</table>";
    
    return html;
}

void hwTable::setCellValue(unsigned int row, unsigned int col, hwWidget* w)
{
    while (myCells.size() < (row + 1))
    {
        vector<hwWidget*> cols;
        myCells.push_back(cols);
    }
    while (myCells[row].size() < (col + 1))
    {
        myCells[row].push_back(NULL);
    }
    myCells[row][col] = w;
}

void hwTable::setColumnLabel(unsigned int col, hwLabel* label)
{
    while (myCols.size() < (col + 1))
    {
        myCols.push_back(NULL);
    }
    myCols[col] = label;
}

hwWidget* hwTable::getCellValue(unsigned int row, unsigned int col)
{
    hwWidget* val = NULL;
    if (myCells.size() && (row < myCells.size()))
    {
        if (myCells[row].size() && (col < myCells[row].size()))
        {
            val = myCells[row][col];
        }
    }
    return val;
}

void hwTable::clear()
{
    myValue = "";
    
    for (unsigned int i = 0; i < myCells.size(); i++)
    {
        for (unsigned int j = 0; j < myCells[i].size(); j++)
        {
            if (myCells[i][j])
            {
                delete myCells[i][j];
            }
        }
        myCells[i].clear();
    }
    myCells.clear();
}

