/*
 * Copyright (c) 2009 Adam Siegel.  All rights reserved.
 *
 * This file is part of libhw.
 *
 * libhw is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with libhw.  If not, see <http: *www.gnu.org/licenses/>.
 *
 */

#include <hwTableLayout.h>

hwTableLayout::hwTableLayout()
        : hwWidget("hwTableLayout")
{
}

hwTableLayout::~hwTableLayout()
{
}

string hwTableLayout::draw()
{
    if (!myVisibleFlag)
    {
        return "";
    }
    string html;
    html = "<table class=\"hwTableLayout\" id=\"" + myName + "\" name=\"" + myName + "\">";
    for (unsigned int i = 0; i <= myNumRows; i++)
    {
        html += "<tr>";
        for (unsigned int j = 0; j <= myNumCols; j++)
        {
            html += "<td>";
            hwWidget* child = getChild(i, j);
            html += child ? child->draw() : "&nbsp;";
            html += "</td>";
        }
        html += "</tr>";
    }
    html += "</table>";
    return html;
}

void hwTableLayout::add(hwWidget* widget, unsigned int row, unsigned int col)
{
    child_t* child = new child_t;
    child->row = row;
    child->col = col;
    child->widget = widget;
    myChildList.push_back(child);
    myNumRows = 0;
    myNumCols = 0;
    vector<child_t*>::iterator iter;
    for (iter = myChildList.begin(); iter != myChildList.end(); iter++)
    {
        if ((*iter)->row > myNumRows)
        {
            myNumRows = (*iter)->row;
        }
        if ((*iter)->col > myNumCols)
        {
            myNumCols = (*iter)->col;
        }
    }
}

hwWidget* hwTableLayout::getChild(unsigned int row, unsigned int col)
{
    vector<child_t*>::iterator iter;
    for (iter = myChildList.begin(); iter != myChildList.end(); iter++)
    {
        if (((*iter)->row == row) && ((*iter)->col == col))
        {
            return (*iter)->widget;
        }
    }
    return NULL;
}

