/*
 * Copyright (c) 2009 Adam Siegel.  All rights reserved.
 *
 * This file is part of libhw.
 *
 * libhw is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with libhw.  If not, see <http: *www.gnu.org/licenses/>.
 *
 */

#include <hwTree.h>
#include <sstream>

hwTree::hwTree()
        : hwWidget("hwTree"), myRoot(NULL)
{
}

void hwTree::addNode(hwWidget* parent, hwWidget* child)
{
    if (parent && child)
    {
        if (!myRoot)
        {
            myRoot = parent;
        }
        
        if (myNodes.find(parent) == myNodes.end())
        {
            vector<hwWidget*> v;
            v.push_back(child);
            myNodes[parent] = v;
        }
        else
        {
            myNodes[parent].push_back(child);
        }
    }
}

string hwTree::draw()
{
    if (!myVisibleFlag)
    {
        return "";
    }
    
    string html;
    
    if (myRoot)
    {
        int count = 0;
        html += "<input type=\"hidden\" name=\"" + myName + "\">";
        html += "<div class=\"dtree\">";
        html += "<script type=\"text/javascript\">";
        html += myName + " = new dTree('" + myName + "');";
        html += myName + ".add(0, -1, '" + myRoot->toString() + "');";
        html += drawNode(myRoot, count, 0);
        html += "document.write(" + myName + ");";
        html += "</script>";
        html += "</div>";
    }
    
    return html;
}

string hwTree::drawNode(hwWidget* node, int& count, int level)
{
    string html;
    
    myNodeLookup[count] = node;
    
    vector<hwWidget*>::iterator iter = myNodes[node].begin();
    
    while (iter != myNodes[node].end())
    {
        stringstream s1;
        s1 << level;
        
        stringstream s2;
        s2 << ++count;

        html += myName + ".add(" + s2.str() + 
            ", " + s1.str() + ", '" + (*iter)->toString() + 
            "', \"javascript:document.forms[0]._widget.value='" +
            myName + "';document.forms[0]._event.value='click';" +
            "document.forms[0]." + myName + ".value='" + s2.str() + 
            "';document.forms[0].submit();top.location='/';\");";
        
        html += drawNode(*iter, count, level + 1);
        
        iter++;
    }
    
    
    return html;
}

