/*
 * Copyright (c) 2009 Adam Siegel.  All rights reserved.
 *
 * This file is part of libhw.
 *
 * libhw is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with libhw.  If not, see <http: *www.gnu.org/licenses/>.
 *
 */

#include <wsServer.h>
#include <wsRequest.h>
#include <iostream>
#include <string.h>

wsServer::wsServer(unsigned int port)
        : myPort(port)
{
}

void wsServer::init()
{
#ifdef WIN32
    WORD ver;
    WSADATA data;
    WSAStartup(ver, &data);
#endif

    int result;
    int one = 1;
    
    memset(&myAddr, 0, sizeof(myAddr));
    
    mySocket = socket(AF_INET, SOCK_STREAM, 0);
    
    setsockopt(mySocket, SOL_SOCKET, SO_REUSEADDR,
               (const char*)&one, sizeof(one));
               
    myAddr.sin_family = AF_INET;
    myAddr.sin_addr.s_addr = INADDR_ANY;
    myAddr.sin_port = htons(myPort);
    
    result = bind(mySocket, (struct sockaddr*)&myAddr, sizeof(myAddr));
    
    if (result == -1)
    {
        cerr << "Unable to bind socket to port " << myPort << endl;
        return;
    }
    
    listen(mySocket, 10);
}

wsRequest* wsServer::getRequest()
{
    int addr_length = sizeof(myAddr);
    int client;
    
#ifdef WIN32
    client = accept(mySocket, (sockaddr*)&myAddr, (int*)&addr_length);
#else
    client = accept(mySocket, (sockaddr*)&myAddr, (socklen_t*)&addr_length);
#endif

    wsRequest* req = new wsRequest(client);
    
    req->process();
    
    return req;
}
