/*
 * Copyright (c) 2009 Adam Siegel.  All rights reserved.
 *
 * This file is part of libhw.
 *
 * libhw is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with libhw.  If not, see <http: *www.gnu.org/licenses/>.
 *
 */

#include <wsStringTokenizer.h>

wsStringTokenizer::wsStringTokenizer(string str, string myDelim)
{
    if ((str.length() == 0) || (myDelim.length() == 0))
    {
        return;
    }
    myTokenStr = str;
    this->myDelim = myDelim;
    unsigned int currentPos = 0;
    while (true)
    {
        if ((currentPos = myTokenStr.find(myDelim,currentPos)) != string::npos)
        {
            currentPos +=myDelim.length();
            while (myTokenStr.find(myDelim,currentPos) == currentPos)
            {
                myTokenStr.erase(currentPos,myDelim.length());
            }
        }
        else
            break;
    }
    if (myTokenStr.find(myDelim,0) == 0)
    {
        myTokenStr.erase(0,myDelim.length());
    }
    currentPos = 0;
    if ((currentPos = myTokenStr.rfind(myDelim)) != string::npos)
    {
        if (currentPos != (myTokenStr.length()-myDelim.length()))
            return;
        myTokenStr.erase(myTokenStr.length()-myDelim.length(),myDelim.length());
    }
}

int wsStringTokenizer::countTokens()
{
    unsigned int prevPos = 0;
    int numTokens = 0;
    if (myTokenStr.length() > 0)
    {
        numTokens = 0;
        unsigned int currentPos = 0;
        while (true)
        {
            if ((currentPos = myTokenStr.find(myDelim,currentPos)) != string::npos)
            {
                numTokens++;
                prevPos     = currentPos;
                currentPos += myDelim.length();
            }
            else
                break;
        }
        return ++numTokens;
    }
    else
    {
        return 0;
    }
}

bool wsStringTokenizer::hasMoreTokens()
{
    return (myTokenStr.length() > 0);
}

string wsStringTokenizer::nextToken()
{
    if (myTokenStr.length() == 0)
    {
        return "";
    }
    string tStr ="";
    unsigned int pos  = myTokenStr.find(myDelim,0);
    if (pos != string::npos)
    {
        tStr     = myTokenStr.substr(0,pos);
        myTokenStr = myTokenStr.substr(pos+myDelim.length(),myTokenStr.length()-pos);
    }
    else
    {
        tStr = myTokenStr.substr(0,myTokenStr.length());
        myTokenStr = "";
    }
    return tStr;
}

int  wsStringTokenizer::nextIntToken()
{
    return atoi(nextToken().c_str());
}

double wsStringTokenizer::nextFloatToken()
{
    return atof(nextToken().c_str());
}

string wsStringTokenizer::nextToken(string myDelimiter)
{
    if (myTokenStr.length() == 0)
    {
        return "";
    }
    string tStr ="";
    unsigned int pos = myTokenStr.find(myDelimiter,0);
    if (pos != string::npos)
    {
        tStr = myTokenStr.substr(0,pos);
        myTokenStr = myTokenStr.substr(pos+myDelimiter.length(),myTokenStr.length()-pos);
    }
    else
    {
        tStr = myTokenStr.substr(0,myTokenStr.length());
        myTokenStr = "";
    }
    return tStr;
}

string wsStringTokenizer::remainingString()
{
    return  myTokenStr;
}

string wsStringTokenizer::filterNextToken(string filterStr)
{
    string str = nextToken();
    unsigned int currentPos = 0;
    while ((currentPos = str.find(filterStr,currentPos)) != string::npos)
    {
        str.erase(currentPos,filterStr.length());
    }
    return str;
}

